# Copyright (c) 2021 John Dewey <john@dewey.ws>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""Kind Driver Module."""

from __future__ import absolute_import

import os

from molecule.api import Driver

from molecule import logger

log = logger.get_logger(__name__)


class Kind(Driver):
    """
    Kind Driver Class.

    The class responsible for managing `kind`_.

    Molecule leverages Ansible's `command`_ module, by mapping
    variables from ``molecule.yml`` into ``create.yml`` and ``destroy.yml``.

    .. code-block:: yaml

        driver:
          name: kind
        platforms:
          - name: molecule-cluster

    .. _`kind`: https://github.com/kubernetes-sigs/kind
    """  # noqa

    def __init__(self, config=None):
        """Construct kind."""
        super(Kind, self).__init__(config)
        self._name = "kind"

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def login_cmd_template(self):
        return (
            "docker exec "
            "-e COLUMNS={columns} "
            "-e LINES={lines} "
            "-e TERM=bash "
            "-e TERM=xterm "
            "-ti {instance}-control-plane bash"
        )

    @property
    def default_safe_files(self):
        return []

    @property
    def default_ssh_connection_options(self):
        return []

    def login_options(self, instance_name):
        return {"instance": instance_name}

    def ansible_connection_options(self, instance_name):
        x = {"ansible_connection": "docker"}
        if "DOCKER_HOST" in os.environ:
            x["ansible_docker_extra_args"] = "-H={}".format(
                os.environ["DOCKER_HOST"]
            )
        return x

    def sanity_checks(self):
        pass

    def reset(self):
        pass



"""Functional tests."""
import os
import subprocess

import molecule.logger
import molecule.util
import pytest
from molecule.test.conftest import change_dir_to
from molecule.test.conftest import random_string  # noqa
from molecule.test.conftest import temp_dir  # noqa

import molecule

LOG = molecule.logger.get_logger(__name__)


def format_result(result: subprocess.CompletedProcess):
    """Return friendly representation of completed process run."""
    return (
        f"RC: {result.returncode}\n"
        + f"STDOUT: {result.stdout}\n"
        + f"STDERR: {result.stderr}"
    )


def test_command_init_scenario(temp_dir, DRIVER):  # noqa
    """Verify that init scenario works."""
    role_directory = os.path.join(temp_dir.strpath, "test-init")
    cmd = [
        "molecule",
        "init",
        "role",
        "test-init",
    ]
    result = molecule.util.run_command(cmd)
    assert result.returncode == 0

    with change_dir_to(role_directory):
        molecule_directory = pytest.helpers.molecule_directory()
        scenario_directory = os.path.join(molecule_directory, "test-scenario")
        options = {"role_name": "test-init", "driver-name": DRIVER}
        cmd = [
            "molecule",
            "init",
            "scenario",
            "test-scenario",
            *molecule.util.dict2args(options),
        ]
        result = molecule.util.run_command(cmd)
        assert result.returncode == 0

        assert os.path.isdir(scenario_directory)

        # TODO(retr0h): Need to correct `verify.yml` upstream.
        # Attempts to always gather facts, which doesn't fit
        # the kind paradigm.
        #  cmd = [
        #      "molecule",
        #      "--debug",
        #      "test",
        #      "-s",
        #      "test-scenario",
        #  ]
        #  result = molecule.util.run_command(cmd)
        #  assert result.returncode == 0



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['molecule_kind', 'molecule_kind.test']

package_data = \
{'': ['*'],
 'molecule_kind': ['cookiecutter/*',
                   'cookiecutter/{{cookiecutter.molecule_directory}}/{{cookiecutter.scenario_name}}/*']}

entry_points = \
{'molecule.driver': ['kind = molecule_kind.driver:Kind']}

setup_kwargs = {
    'name': 'molecule-kind',
    'version': '0.1.0',
    'description': 'molecule-kind - Molecule Kind Driver allows Molecule users to test Ansible code using Kind.',
    'long_description': '# molecule-kind\n\n[![Unit Test](https://github.com/retr0h/molecule-kind/actions/workflows/unit.yml/badge.svg)](https://github.com/retr0h/molecule-kind/actions/workflows/unit.yml)\n[![Lint](https://github.com/retr0h/molecule-kind/actions/workflows/lint.yml/badge.svg)](https://github.com/retr0h/molecule-kind/actions/workflows/lint.yml)\n\nmolecule-kind - Molecule Kind Driver allows Molecule users to test Ansible code using Kind.\n\n## Dependencies\n\n* [kind][]\n\n## Installing\n\n    $ pip install molecule-kind\n\n## Usage\n\n    $ molecule init scenario -d kind\n    $ molecule test\n\n## Testing\n\nTo execute unit tests.\n\n    $ make dep\n    $ make test\n\n## License\n\nThe [MIT] License.\n\n[kind]: https://github.com/kubernetes-sigs/kind\n[MIT]: LICENSE\n',
    'author': 'John Dewey',
    'author_email': 'john@dewey.ws',
    'maintainer': 'John Dewey',
    'maintainer_email': 'john@dewey.ws',
    'url': 'https://github.com/retr0h/molecule-kind',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
